@@ -4,7 +4,7 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
-from plugins.events import EventPlugin
+from quodlibet.plugins.events import EventPlugin
 
 class IRadioLog(EventPlugin):
     PLUGIN_ID = "Internet Radio Log"
