@@ -10,6 +10,7 @@ import sys
 
 import gtk
 import pango
+import gobject
 
 gtk_216 = gtk.gtk_version >= (2, 16)
 if not gtk_216:
@@ -18,25 +19,19 @@ if not gtk_216:
     except ImportError:
         import _trayicon as trayicon
 
-from quodlibet import browsers
-from quodlibet import config
-from quodlibet import const
-from quodlibet import qltk
-from quodlibet import stock
-from quodlibet import util
+from gtk.gdk import SCROLL_LEFT, SCROLL_RIGHT, SCROLL_UP, SCROLL_DOWN
+
+from quodlibet import browsers, config, const, qltk, stock, util
+from quodlibet.library import library
 from quodlibet.parse import Pattern
+from quodlibet.player import playlist as player
 from quodlibet.plugins.events import EventPlugin
 from quodlibet.qltk.browser import LibraryBrowser
-from quodlibet.qltk.information import Information
-from quodlibet.qltk.properties import SongProperties
 from quodlibet.qltk.controls import StopAfterMenu
-
-from quodlibet.player import playlist as player
-from quodlibet.widgets import main as window, watcher
+from quodlibet.qltk.information import Information
 from quodlibet.qltk.playorder import ORDERS
-from quodlibet.library import library
-
-from gtk.gdk import SCROLL_LEFT, SCROLL_RIGHT, SCROLL_UP, SCROLL_DOWN
+from quodlibet.qltk.properties import SongProperties
+from quodlibet.widgets import main as window, watcher
 
 class Preferences(gtk.VBox):
     """A small window to configure the tray icon's tooltip."""
@@ -205,10 +200,11 @@ class TrayIcon(EventPlugin):
     __pixbuf = None
     __pixbuf_paused = None
     __icon_theme = None
-    __position = None
+    __menu = None
     __size = -1
     __w_sig_map = None
     __w_sig_del = None
+    __theme_sig = None
     __stop_after = None
     __first_map = True
     __pattern = Pattern(
@@ -229,7 +225,8 @@ class TrayIcon(EventPlugin):
             self.__icon =  EggTrayIconWrapper()
 
         self.__icon_theme = gtk.icon_theme_get_default()
-        self.__icon_theme.connect('changed', self.__theme_changed)
+        self.__theme_sig = self.__icon_theme.connect('changed',
+            self.__theme_changed)
 
         self.__icon.connect('size-changed', self.__size_changed)
         #no size-changed under win32
@@ -251,18 +248,15 @@ class TrayIcon(EventPlugin):
         self.plugin_on_song_started(player.song)
 
     def disabled(self):
-        global gtk_216
-        if self.__w_sig_map and window.handler_is_connected(self.__w_sig_map):
-            window.disconnect(self.__w_sig_map)
-        if self.__w_sig_del and window.handler_is_connected(self.__w_sig_del):
-            window.disconnect(self.__w_sig_del)
-        if self.__icon:
-            self.__icon.set_visible(False)
-            try:
-                self.__icon.destroy()
-            except AttributeError:
-                pass
-            self.__icon = None
+        self.__icon_theme.disconnect(self.__theme_sig)
+        self.__icon_theme = None
+        self.__stop_after = None
+        window.disconnect(self.__w_sig_map)
+        window.disconnect(self.__w_sig_del)
+        self.__icon.set_visible(False)
+        try: self.__icon.destroy()
+        except AttributeError: pass
+        self.__icon = None
         self.__show_window()
 
     def PluginPreferences(self, parent):
@@ -274,7 +268,7 @@ class TrayIcon(EventPlugin):
         if self.__size <= 0:
             return
 
-        pixbuf_size = int(self.__size * 0.75)
+        pixbuf_size = max(int(self.__size * 0.75), 1)
         #windows panel has enough padding
         if sys.platform == "win32":
             pixbuf_size = self.__size
@@ -285,29 +279,40 @@ class TrayIcon(EventPlugin):
             try:
                 self.__pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(
                     filename + "svg", pixbuf_size * 2, pixbuf_size * 2)
-            except:
+            except gobject.GError:
                 self.__pixbuf = gtk.gdk.pixbuf_new_from_file(filename + "png")
             self.__pixbuf = self.__pixbuf.scale_simple(
                 pixbuf_size, pixbuf_size, gtk.gdk.INTERP_BILINEAR)
 
         if player.paused:
             if not self.__pixbuf_paused:
-                base = self.__pixbuf.copy()
-                overlay = self.__icon_theme.load_icon(
-                    gtk.STOCK_MEDIA_PAUSE,
-                    pixbuf_size, gtk.ICON_LOOKUP_FORCE_SVG)
+                overlay = None
+                # basic Kubuntu 10.10 has got no gtk.STOCK_MEDIA_PAUSE
+                icon_names = (gtk.STOCK_MEDIA_PAUSE, 'media-playback-pause')
+                for name in icon_names:
+                    icon_info = self.__icon_theme.lookup_icon(
+                        name, pixbuf_size, 0)
+                    if icon_info is not None:
+                        try: overlay = icon_info.load_icon()
+                        except gobject.GError: pass
+                        else: break
 
-                w, h = base.get_width(), base.get_height()
-                wo, ho = overlay.get_width(), overlay.get_height()
-                r = 2
-                b = 8
-                l = b - r
+                if overlay is not None:
+                    base = self.__pixbuf.copy()
+                    w, h = base.get_width(), base.get_height()
+                    wo, ho = overlay.get_width(), overlay.get_height()
+                    r = 2
+                    b = 8
+                    l = b - r
+
+                    overlay.composite(base, w * r // b, h * r // b,
+                        l * w // b, l * h // b,
+                        w * r // b, h * r // b + 1,
+                        float(l * w) / b / wo, float(l * h) / b / ho,
+                        gtk.gdk.INTERP_BILINEAR, 255)
+                else:
+                    base = self.__pixbuf
 
-                overlay.composite(base, w * r // b, h * r // b,
-                    l * w // b, l * h // b,
-                    w * r // b, h * r // b + 1,
-                    float(l * w) / b / wo, float(l * h) / b / ho,
-                    gtk.gdk.INTERP_BILINEAR, 255)
                 self.__pixbuf_paused = base
 
             new_pixbuf = self.__pixbuf_paused
@@ -359,18 +364,15 @@ class TrayIcon(EventPlugin):
 
     def __hide_window(self):
         self.__first_map = False
-        self.__position = window.get_position()
         window.hide()
         config.set("plugins", "icon_window_visible", "false")
 
     def __show_window(self):
-        if self.__position:
-            window.move(*self.__position)
-
         window.show()
         window.present()
 
     def __button_left(self, icon):
+        if self.__destroy_win32_menu(): return
         if window.get_property('visible'):
             self.__hide_window()
         else:
@@ -378,6 +380,7 @@ class TrayIcon(EventPlugin):
 
     def __button_middle(self, widget, event):
         if event.type == gtk.gdk.BUTTON_PRESS and event.button == 2:
+            if self.__destroy_win32_menu(): return
             self.__play_pause()
 
     def __play_pause(self, *args):
@@ -407,6 +410,8 @@ class TrayIcon(EventPlugin):
                 player.volume -= 0.05
 
     def plugin_on_song_started(self, song):
+        if not self.__icon: return
+
         if song:
             try:
                 pattern = Pattern(config.get("plugins", "icon_tooltip"))
@@ -419,9 +424,18 @@ class TrayIcon(EventPlugin):
 
         self.__icon.set_tooltip(tooltip)
 
+    def __destroy_win32_menu(self):
+        """Returns True if current action should only hide the menu"""
+        if sys.platform == "win32" and self.__menu:
+            self.__menu.destroy()
+            self.__menu = None
+            return True
+
     def __button_right(self, icon, button, time):
         global gtk_216
-        menu = gtk.Menu()
+
+        if self.__destroy_win32_menu(): return
+        self.__menu = menu = gtk.Menu()
 
         pp_icon = [gtk.STOCK_MEDIA_PAUSE, gtk.STOCK_MEDIA_PLAY][player.paused]
         playpause = gtk.ImageMenuItem(pp_icon)
@@ -439,7 +453,7 @@ class TrayIcon(EventPlugin):
         repeat.connect('toggled',
             lambda s: window.repeat.set_active(s.get_active()))
 
-        
+
         def set_safter(widget, stop_after):
             stop_after.active = widget.get_active()
 
@@ -518,7 +532,9 @@ class TrayIcon(EventPlugin):
 
         menu.show_all()
 
-        if gtk_216:
+        if sys.platform == "win32":
+            menu.popup(None, None, None, button, time, self.__icon)
+        elif gtk_216:
             menu.popup(None, None, gtk.status_icon_position_menu,
                 button, time, self.__icon)
         else:
@@ -534,6 +550,3 @@ class TrayIcon(EventPlugin):
     def __information(self, *args):
         if player.song:
             Information(watcher, [player.song])
-
-    def destroy(self):
-        self.disabled()
